@@ -8,72 +8,55 @@ use warnings;
 
 use overload ();
 
-our $VERSION = 0.02;
+our $VERSION = 0.03;
 
-use Scalar::Util qw/weaken/;
+use Scalar::Util qw/weaken reftype/;
+use Variable::Magic qw/wizard cast/;
 
-sub purge {
-	my $self = shift;
-	$self->[3] = 0;
+my $wiz = wizard free => \&_clear_weakened_sub, data => sub { my ( $var, $self ) = @_; $self };
 
-	foreach my $key (keys %{ $self->[0] }){
-		delete $self->[0]{$key} unless defined $self->[0]{$key}[0]; # delete if the reference became undef
-	}
+sub _clear_weakened_sub {
+	my ( $key, $self ) = @_;
+	$self->_clear_weakened($key) if defined $self; # support subclassing
 }
 
-sub maybe_purge {
-	my $self = shift;
-	$self->purge if ++$self->[3] > (10 + ($self->[4] ||= 15) * log(scalar keys %{ $self->[0] } || 1));
+sub _clear_weakened {
+	my ( $self, $key ) = @_;
+
+	$self->DELETE( $key );
 }
 
 sub STORE {
 	my($s, $k, $v) = @_;
-	$s->maybe_purge;
 
 	if (ref $k) {
 		# make sure we use the same function that RefHash is using for ref keys
 		my $kstr = Tie::RefHash::refaddr($k);
-		weaken(($s->[0]{$kstr} = [$k, $v])->[0]);
+		my $entry = [$k, $v];
+
+		weaken( $entry->[0] );
+
+		if ( reftype $k eq 'SCALAR' ) {
+			cast $$k, $wiz, $s;
+		} elsif ( reftype $k eq 'HASH' ) {
+			cast %$k, $wiz, $s;
+		} elsif ( reftype $k eq 'ARRAY' ) {
+			cast @$k, $wiz, $s;
+		} elsif ( reftype $k eq 'GLOB' ) {
+			cast *{$k}, $wiz, $s;
+		} else {
+			die "patches welcome";
+		}
+
+		$s->[0]{$kstr} = $entry;
 	}
 	else {
 		$s->[1]{$k} = $v;
 	}
+
 	$v;
 }
 
-sub FETCH {
-	my ($s, $k) = @_;
-	$s->maybe_purge;
-	$s->SUPER::FETCH($k);
-}
-
-sub EXISTS {
-	my ($s, $k) = @_;
-	$s->maybe_purge;
-	$s->SUPER::EXISTS($k);
-}
-
-sub NEXTKEY {
-	my $s = shift;
-
-	my ($k, $v);
-
-	if (!$s->[2]){
-		while (($k, $v) = each %{ $s->[0] }) {
-			if (defined $v->[0]){ # weak refs wiuth no referants (thingys) become undef
-				$s->[3]-- if $s->[3] > 0;
-				return $v->[0];
-			} else {
-				delete $s->[0]{$k};
-			}
-		}
-
-		$s->[2] = 1; # we're out of references, so lets iterate the "regular" hash.
-	}
-
-	return each %{ $s->[1] };
-}
-
 __PACKAGE__
 
 __END__
@@ -118,60 +101,6 @@ This subclass of L<Tie::RefHash> has weak keys, instead of strong ones. The
 values are left unaltered, and you'll have to make sure there are no strong
 references there yourself.
 
-=head1 STRUCTURE MAINTAINCE
-
-In perl when weak refernces go bad, they become undef. L<Tie::RefHash::Weak>
-needs to occasionally go through it's structures and clean out the stale keys.
-
-It does this on two occasions:
-
-=over 4
-
-=item As necessary
-
-When an aggregate operation, like L<keys> and L<values> (even in scalar
-context), or list context interpolation of the hash is made, the whole hash's
-state must be validated.
-
-This is done lazily, within each call to L<NEXTKEY>. If you need to ensure a
-low latency, then only iterate with L<each>.
-
-=item Occasionally
-
-A counter is raised on each update. It's stored in C<$self->[3]>. The counter
-is incremented via the C<maybe_purge> method. If the counter is bigger than ten
-plus C<$self->[4]> (which defaults to 15) times the natural logarithm of the
-number of actual reference keys in the hash (before we know how many are
-stale), then the hash is iterated and purged. This value tries to ensure that
-purges aren't made too often, so that they don't take up too much time, but the
-cleanup is still made in order to prevent leaking data, and to decrement the
-reference counts of the I<values> of the hash.
-
-If you know that your hash will not be accessed often, and 
-
-=back
-
-You can disable purging by making C<maybe_purge> a no-op in your subclass.
-
-You can forcibly purge by calling
-
-	(tied %hash)->purge;
-
-You can tweak the occasional purging by playing setting
-
-	(tied %hash)->[4] = $x;
-
-=head1 TODO
-
-=over 4
-
-=item Hook Perl_magic_killbackrefs
-
-Maybe some day the code which undefines weak references will be hooked with XS
-to purge the hash in a more real-time fashion.
-
-=back
-
 =head1 THREAD SAFETY
 
 L<Tie::RefHash> version 1.32 and above have correct handling of threads (with
